@@ -1,7 +1,9 @@
 package Catalyst::Plugin::Shorten;
-use 5.006; use strict; use warnings; our $VERSION = '0.01';
+use 5.006; use strict; use warnings; our $VERSION = '0.02';
 
 use Bijection qw/all/;
+use Scalar::Util qw/reftype/;
+use Carp;
 
 our ($pkey, $ukey, $short);
 
@@ -14,6 +16,10 @@ sub setup {
 		$c->shorten_bijection_set(@{$config->{set}});
 	}
 
+	if ($config->{offset}) {
+		$c->shorten_offset_set($config->{offset});
+	}
+
 	$pkey = $config->{map}->{params} || 'params';
 	$ukey = $config->{map}->{uri} || 'uri';
 	$short = $config->{map}->{s} || 's';
@@ -22,66 +28,81 @@ sub setup {
 }
 
 sub shorten {
-	my ($c, $asUrl, $additional) = @_;
+	my ($c, %args) = @_;
 
 	my $id = $c->shorten_set_data(
-		$pkey => $c->req->params,
-		$ukey => $c->req->uri,
-		%{$additional}
+		($args{uri}
+			? (
+				$ukey => $args{uri},
+				$pkey => $args{uri}->query_form_hash
+			)
+			: (
+				$ukey => $c->req->uri,
+				$pkey => $c->req->params,
+			)
+		),
+		($args{store} ? %{$args{store}} : ())
 	);
 
 	my $string = biject($id);
-	return $asUrl
-		? $c->uri_for_action($c->action->private_path, { $short => $string })
+	return $args{as_uri}
+		? $args{uri}
+			? do { $args{uri}->query_form($short => $string); $args{uri} }
+			: $c->uri_for_action($c->action->private_path, { $short => $string })
 		: $string;
 }
 
 sub shorten_delete {
-	my ($c, $str) = @_;
-	if (my $id = inverse($str)) {
-		return $c->shorten_delete_data($id);
+	my ($c, %args) = @_;
+
+	if (!$args{$short}) {
+		unless ($args{$short} = $c->req->param($short)) {
+			Catalyst::Exception->throw(sprintf 'Unable to find %s to delete', $short);		
+		}
 	}
+
+	my $id = inverse($args{$short});
+	return $c->shorten_delete_data($id);
 }
 
 sub shorten_extract {
-	my ($c, $cb) = @_;
+	my ($c, %args) = @_;
 
-	my $params = $c->req->params;
-	my $sparams = $c->shorten_params($params, $cb);
-
-	if ($sparams) {
-		delete $params->{$short};
-		$sparams = {%{$sparams}, %{$params}};
+	$args{params} = $c->req->params unless $args{params}; # 100% the conditionals, ||= would always be true :)
+	if (my $sparams = $c->shorten_params(%args)) {
+		delete $args{params}->{$short};
+		$sparams = {%{$sparams}, %{$args{params}}} unless $args{no_merge};
 		$c->req->parameters($sparams);
 	}
 
-	1;
+	1;	
 }
 
 sub shorten_params {
-	my ($c, $params, $cb) = @_;
-	$params ||= $c->req->params;
-	if ($params->{$short}) {
-		if (my $id = inverse($params->{$short})) {
-			my $shorten = $c->shorten_get_data($id);
-			if (defined $cb) {
-				return $cb->($c, $shorten->{$pkey}, $shorten);
-			}
-			return $shorten->{$pkey};
+	my ($c, %args) = @_;
+	$args{params} = $c->req->params unless $args{params};
+	if ($args{params}->{$short}) {
+		my $id = inverse($args{params}->{$short});
+		my $shorten = $c->shorten_get_data($id);
+		if (exists $args{cb}) {
+			$shorten = $args{cb}->($c, $shorten);
 		}
+		return $shorten->{$pkey} if $shorten; # reftype for blessed
+		Catalyst::Exception->throw(sprintf 'Unable to find params for: %s -> %s : %s', 
+			$args{params}->{$short}, $id, $pkey);
+	
 	}
 }
 
 sub shorten_redirect {
-	my ($c, $str, $cb) = @_;
-	if (my $id = inverse($str)) {
-		my $shorten = $c->shorten_get_data($id);
-		my $uri = $shorten->{$ukey};
-		if (defined $cb) {
-			$uri = $cb->($c, $shorten->{$ukey}, $shorten);
-		}
-		$c->res->redirect($uri) if $uri;
+	my ($c, %args) = @_;
+	my $id = inverse($args{$short});
+	my $shorten = $c->shorten_get_data($id);
+	if (exists $args{cb}) {
+		$shorten = $args{cb}->($c, $shorten);
 	}
+	return $c->res->redirect($shorten->{$ukey}) if $shorten;
+	Catalyst::Exception->throw(sprintf 'Unable to find uri to redirect for: %s -> %s', $args{$short}, $id);	
 }
 
 sub shorten_bijection_set {
@@ -89,6 +110,11 @@ sub shorten_bijection_set {
 	bijection_set(@set);
 }
 
+sub shorten_offset_set {
+	my ($c, @set) = @_;
+	offset_set(@set);
+}
+
 1;
 
 __END__
@@ -97,9 +123,14 @@ __END__
 
 Catalyst::Plugin::Shorten - The great ancient URI shortner!
 
+=for html
+<a href="https://travis-ci.org/ThisUsedToBeAnEmail/Catalyst-Plugin-Shorten"><img src="https://travis-ci.org/ThisUsedToBeAnEmail/Catalyst-Plugin-Shorten.svg?branch=master" alt="Build Status"></a>
+<a href="https://coveralls.io/r/ThisUsedToBeAnEmail/Catalyst-Plugin-Shorten?branch=master"><img src="https://coveralls.io/repos/ThisUsedToBeAnEmail/Catalyst-Plugin-Shorten/badge.svg?branch=master" alt="Coverage Status"></a>
+<a href="https://metacpan.org/pod/Catalyst-Plugin-Shorten"><img src="https://badge.fury.io/pl/Catalyst-Plugin-Shorten.svg" alt="CPAN version"></a>
+
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
@@ -121,7 +152,7 @@ Version 0.01
 		my ($self, $c) = @_;
 
 		my $str = $c->shorten(); # returns bijection references to an ID in the store.
-		my $url = $c->shorten(1); # return a url to the current endpoint replacing all params with localhost:300/ending?s=GH
+		my $url = $c->shorten(as_uri => 1); # return a url to the current endpoint replacing all params with localhost:300/ending?s=GH
 	}
 	
 	-------
@@ -138,6 +169,7 @@ Version 0.01
 			map => {
 				params => 'data',
 				uri => 'url',
+				s => 'g'
 			}
 		}
 	);
@@ -153,7 +185,7 @@ Version 0.01
 		
 	sub g :Chained('/') :PathPart('g') :Args('1') {
 		my ($self, $c, $cap) = @_;
-		$c->shorten_redirect($cap);
+		$c->shorten_redirect(g => $cap);
 	}
 
 	__PACKAGE__->meta->make_immutable;
@@ -162,19 +194,19 @@ Version 0.01
 
 =head1 SUBROUTINES/METHODS
 
-=head2 shorten (asUrl, $additional)
+=head2 shorten (as_uri => 1|0, uri => URI, store => {} )
 
 Take the current request uri and store, returns an Bijective string.
 
 =cut
 
-=head2 shorten_delete (str)
+=head2 shorten_delete (s => '')
 
 Delete from storage.
 
 =cut
 
-=head2 shorten_extract (cb)
+=head2 shorten_extract (params => { s => ...}, cb => sub)
 
 Check for the param (default is 's'), if defined attempt to inverse and then right merge with the current requests params.
 
@@ -184,18 +216,30 @@ This always returns true and you can later access the merged params using -
 
 =cut
 
-=head2 shorten_params (params, cb)
+=head2 shorten_params (params => { s => ...}, cb => sub)
 
 Check for the param (default is 's'), if defined attempt to inverse and then return the params retrieved from storage.
 
 =cut
 
-=head2 shorten_redirect (str, cb)
+=head2 shorten_redirect (s => '', cb => sub)
 
 Redirect the clients browser to the uri retrieved from the storage.
 
 =cut
 
+=head2 shorten_bijection_set (@set)
+
+=cut
+
+=head2 shorten_offset_set ($offset)
+
+=cut
+
+=head2 setup
+
+=cut
+
 =head1 AUTHOR
 
 LNATION, C<< <thisusedtobeanemail at gmail.com> >>
